/**
* @license Copyright (c) Microsoft Corporation.  All rights reserved.
*/
/// <reference path="../../../../TypeDefinitions/mscrm.d.ts" />
/// <reference path="CommonReferences.ts" />
/// <reference path="../../../../../references/external/TypeDefinitions/jquery.d.ts" />
/// <reference path="../../../../../references/external/TypeDefinitions/lib.es6.d.ts"/> 
/**
* @license Copyright (c) Microsoft Corporation.  All rights reserved.
*/
/// <reference path="privatereferences.ts"/>
var MscrmControls;
(function (MscrmControls) {
    var USD;
    (function (USD) {
        var Controls;
        (function (Controls) {
            'use strict';
            var UserSettingsControl = (function () {
                /**
                 * Constructor.
                 */
                function UserSettingsControl() {
                    this._context = null;
                    this._selectedAttributesList = [];
                    this._systemuserAttributesList = [];
                    this._mapLogicalNameToDisplayName = new Map();
                    this._mapLogicalNameToUniqueId = new Map();
                    this._mapUniqueIdToLogicalName = new Map();
                    this._odataResponseReceived = false;
                    this._updatingForFirstTime = true;
                }
                /**
                 * This function should be used for any initial setup necessary for your control.
                 * @params context The "Input Bag" containing the parameters and other control metadata.
                 * @params notifyOutputchanged The method for this control to notify the framework that it has new outputs
                 * @params state The user state for this control set from setState in the last session
                 * @params container The div element to draw this control in
                 */
                UserSettingsControl.prototype.init = function (context, notifyOutputChanged, state, container) {
                    // custom code goes here
                    this._context = context;
                    this._notifyOutputChanged = notifyOutputChanged || (function () { });
                    if (this.isCRMUCI())
                        this.retrieveUserAttributes();
                };
                /**
                 * This function will recieve an "Input Bag" containing the values currently assigned to the parameters in your manifest
                 * It will send down the latest values (static or dynamic) that are assigned as defined by the manifest & customization experience
                 * as well as resource, client, and theming info (see mscrm.d.ts)
                 * @params context The "Input Bag" as described above
                 */
                UserSettingsControl.prototype.updateView = function (context) {
                    var _this = this;
                    var container;
                    if (!this.isCRMUCI())
                        return container;
                    if (this._odataResponseReceived) {
                        //parse input and populated necessary local objects
                        this._input = this._context.parameters.userSettingsControlInput.raw;
                        if (!this.isNullOrUndefined(this._input) && this._updatingForFirstTime) {
                            this._updatingForFirstTime = false;
                            var xmlDocument = (new DOMParser()).parseFromString(this._input, "text/xml");
                            this._selectedAttributesList = Array.prototype.map.call(xmlDocument.getElementsByTagName("option"), function (optionNode) {
                                if (!_this.isNullOrUndefined(optionNode)
                                    && !_this.isNullOrUndefined(optionNode.getAttribute("value"))) {
                                    return optionNode.getAttribute("value");
                                }
                            });
                        }
                        return this.getDualListControlComponent();
                    }
                    else {
                        var loadingLabel = this._context.factory.createElement("LABEL", {
                            key: Controls.UserSchemaControlConstants.LOADING_LABEL_ID,
                            id: Controls.UserSchemaControlConstants.LOADING_LABEL_ID,
                            tabIndex: 0,
                            accessibilityLabel: this._context.resources.getString(Controls.UserSchemaControlConstants.LOADINGLABEL_STRING_KEY)
                        }, this._context.resources.getString(Controls.UserSchemaControlConstants.LOADINGLABEL_STRING_KEY));
                        container = this._context.factory.createElement("CONTAINER", {
                            key: Controls.UserSchemaControlConstants.LOADINGLABEL_CONTAINER_ID,
                            id: Controls.UserSchemaControlConstants.LOADINGLABEL_CONTAINER_ID
                        }, [loadingLabel]);
                    }
                    return container;
                };
                /**
                 * This function will return an "Output Bag" to the Crm Infrastructure
                 * The ouputs will contain a value for each property marked as "input-output"/"bound" in your manifest
                 * i.e. if your manifest has a property "value" that is an "input-output", and you want to set that to the local variable "myvalue" you should return:
                 * {
                 *		value: myvalue
                 * };
                 * @returns The "Output Bag" containing values to pass to the infrastructure
                 */
                UserSettingsControl.prototype.getOutputs = function () {
                    // custom code goes here - remove the line below and return the correct output
                    var output = {
                        userSettingsControlOutput: this.generateOutputXml(this._selectedAttributesList)
                    };
                    return output;
                };
                /**
                 * This function will be called when the control is destroyed
                 * It should be used for cleanup and releasing any memory the control is using
                 */
                UserSettingsControl.prototype.destroy = function () {
                };
                /**
                 * Function checks if control is loaded on UCI or any other client
                 */
                UserSettingsControl.prototype.isCRMUCI = function () {
                    return this._context.utils.isNullOrUndefined(Xrm.Internal.isUci) ? false : Xrm.Internal.isUci();
                };
                /**
                 * This function retrieves the data needed by control to render.
                 */
                UserSettingsControl.prototype.retrieveUserAttributes = function () {
                    var _this = this;
                    var attributesMetadataWebApiURL = this._context.utils.createCrmUri(this.getAttributesOfEntityURL("systemuser"));
                    this.sendRequest(attributesMetadataWebApiURL).then(function (response) {
                        var attributeList = response.value;
                        attributeList.forEach(function (attribute, index) {
                            var attributeLogicalName = attribute.LogicalName;
                            _this._systemuserAttributesList.push(attributeLogicalName);
                            _this._mapLogicalNameToDisplayName.set(attributeLogicalName, attribute.DisplayName);
                            _this._mapLogicalNameToUniqueId.set(attributeLogicalName, index);
                            _this._mapUniqueIdToLogicalName.set(index, attributeLogicalName);
                        });
                        _this._odataResponseReceived = true;
                        _this._context.utils.requestRender();
                    }, function (error) {
                        var errorMessage = "OData call for attribute metadata retrieval of Systemuser entity failed.";
                        console.error(errorMessage);
                        var alertMessage = {
                            text: _this._context.resources.getString(Controls.UserSchemaControlConstants.WEBAPIREQUESTFAILED_ERROR),
                            confirmButtonLabel: _this._context.resources.getString(Controls.UserSchemaControlConstants.CONFIRMBUTTON_TEXT)
                        };
                        _this._context.navigation.openAlertDialog(alertMessage);
                    });
                };
                /**
                 * This method generates odata URL to fetch entity attribute metadata with certain conditions.
                 * @param entityName name of the entity.
                 */
                UserSettingsControl.prototype.getAttributesOfEntityURL = function (entityName) {
                    return "/api/data/v9.0/EntityDefinitions(LogicalName='" + entityName + "')/Attributes?$filter=IsLogical eq false&$select=LogicalName,DisplayName";
                };
                /**
                * This method is used to send webapi request with the specified url
                * @param url The url paramemer.
                */
                UserSettingsControl.prototype.sendRequest = function (url) {
                    return $.ajax({
                        url: encodeURI(url),
                        type: 'GET',
                        async: true,
                        contentType: "application/json",
                        dataType: "json"
                    });
                };
                /**
                 * This function returns dual list control component to render with data.
                 */
                UserSettingsControl.prototype.getDualListControlComponent = function () {
                    var properties = {
                        "parameters": {
                            JsonOptions: {
                                Usage: 3,
                                Static: true,
                                Type: "SingleLine.Text",
                                Value: this.getAllAttributeListForDualListControlRendering(),
                                Attributes: {
                                    DisplayName: null,
                                    LogicalName: "all_attributes",
                                    Type: "string",
                                    IsSecured: false,
                                    RequiredLevel: 0,
                                    MaxLength: 2147483647,
                                    EntityLogicalName: "",
                                    Format: "text",
                                    ImeMode: -1,
                                    Behavior: null
                                }
                            },
                            Selection: {
                                Usage: 3,
                                Static: true,
                                Type: "SingleLine.Text",
                                Value: this.getSelectionString(this._selectedAttributesList),
                                Callback: this.callBackOnDualListSelectionChange.bind(this),
                                Attributes: {
                                    DisplayName: null,
                                    LogicalName: "cc_selectedAttributes_id",
                                    Type: "string",
                                    IsSecured: false,
                                    RequiredLevel: 0,
                                    MaxLength: 2147483647,
                                    EntityLogicalName: "",
                                    Format: null,
                                    ImeMode: -1,
                                    Behavior: null
                                }
                            },
                            SelectionOrderBy: {
                                Usage: 1,
                                Static: true,
                                Type: "Enum",
                                Value: 3,
                                Primary: false
                            },
                            OptionSet: {
                                Usage: 3,
                                Static: true,
                                Type: null,
                                attributes: {}
                            }
                        }
                    };
                    var dualListContainer = this._context.factory.createElement("CONTAINER", {
                        id: Controls.UserSchemaControlConstants.DUALLIST_CONTAINER_ID,
                        key: Controls.UserSchemaControlConstants.DUALLIST_CONTAINER_ID,
                        style: {
                            display: "flex",
                            flex: "1",
                            alignItems: "center",
                            justifyContent: "center",
                            overflow: "auto"
                        }
                    }, this._context.factory.createComponent("MscrmControls.DualListSelection.DualListSelectionControl", "DualListSelectionControl", properties));
                    return dualListContainer;
                };
                /**
                 * Get localized name corresponding to logical name of attribute.
                 * @param logicalName logical name of the attribute.
                 */
                UserSettingsControl.prototype.getLocalizedDisplayNameFromLogicalName = function (logicalName) {
                    return this._mapLogicalNameToDisplayName.get(logicalName).UserLocalizedLabel.Label;
                };
                /**
                 * This function generates attribute list in format required by dual list control.
                 */
                UserSettingsControl.prototype.getAllAttributeListForDualListControlRendering = function () {
                    var _this = this;
                    //First sort the array
                    this._systemuserAttributesList = this._systemuserAttributesList.sort(function (logicalName1, logicalName2) {
                        var displayName1 = _this.getLocalizedDisplayNameFromLogicalName(logicalName1);
                        var displayName2 = _this.getLocalizedDisplayNameFromLogicalName(logicalName2);
                        return displayName1.localeCompare(displayName2);
                    });
                    //Convert it to format required by dual list control
                    var dualListAllAttributeInput = this._systemuserAttributesList.map(function (logicalName) {
                        return {
                            Id: _this._mapLogicalNameToUniqueId.get(logicalName),
                            DisplayName: _this.getLocalizedDisplayNameFromLogicalName(logicalName)
                        };
                    });
                    return JSON.stringify(dualListAllAttributeInput);
                };
                /**
                 * Generates selected attributes string in format required by dual list control
                 * @param selectedAttributesList list of selected attributes
                 */
                UserSettingsControl.prototype.getSelectionString = function (selectedAttributesList) {
                    var _this = this;
                    return selectedAttributesList.reduce(function (selectionString, selectedAttribute) { return selectionString + _this._mapLogicalNameToUniqueId.get(selectedAttribute) + ","; }, "");
                };
                /**
                 * This function gets executed when there is selection change in dual list control.
                 * @param newlySelectedValues newly selected values in dual list control.
                 */
                UserSettingsControl.prototype.callBackOnDualListSelectionChange = function (newlySelectedValues) {
                    var _this = this;
                    this._selectedAttributesList = newlySelectedValues.split(',')
                        .filter(function (idString) { if (idString.trim() != "")
                        return idString; })
                        .map(function (idString) {
                        return _this._mapUniqueIdToLogicalName.get(Number(idString));
                    });
                    this._notifyOutputChanged();
                    this._context.utils.requestRender();
                };
                /**
                 * Generates output for this control from selected attributes.
                 * @param selectedAttributesList list of attributes which are selected.
                 */
                UserSettingsControl.prototype.generateOutputXml = function (selectedAttributesList) {
                    /*
                    Output Data should be in following format. This is stored in DB.
                        <select>
                            <option value='logicalNameOfAttribute1'/>
                            <option value='logicalNameOfAttribute2'/>
                            ...
                        </select>
                    */
                    var selectNode = document.createElement("select");
                    var rootNode = document.createElement("root");
                    rootNode.appendChild(selectNode);
                    selectedAttributesList.forEach(function (selectedAttribute, index) {
                        var valueAttribute = document.createAttribute("value");
                        valueAttribute.value = selectedAttribute;
                        var optionNode = document.createElement("option");
                        optionNode.setAttributeNode(valueAttribute);
                        selectNode.appendChild(optionNode);
                    });
                    return rootNode.innerHTML;
                };
                /**
                 * This method checks if object is null or undefined.
                 * @param object object to be checked
                 */
                UserSettingsControl.prototype.isNullOrUndefined = function (object) {
                    return this._context.utils.isNullOrUndefined(object);
                };
                return UserSettingsControl;
            }());
            Controls.UserSettingsControl = UserSettingsControl;
        })(Controls = USD.Controls || (USD.Controls = {}));
    })(USD = MscrmControls.USD || (MscrmControls.USD = {}));
})(MscrmControls || (MscrmControls = {}));
/**
* @license Copyright (c) Microsoft Corporation. All rights reserved.
*/
/// <reference path="inputsoutputs.g.ts" />
/// <reference path="UserSettingsControl.ts" />
var MscrmControls;
(function (MscrmControls) {
    var USD;
    (function (USD) {
        var Controls;
        (function (Controls) {
            var UserSchemaControlConstants;
            (function (UserSchemaControlConstants) {
                UserSchemaControlConstants.DUALLIST_CONTAINER_ID = "userschemacontrol_duallistcontainer";
                UserSchemaControlConstants.LOADING_LABEL_ID = "userschemacontrol_loadinglabel";
                UserSchemaControlConstants.LOADINGLABEL_CONTAINER_ID = "userschemacontrol_loadinglabelcontainer";
                UserSchemaControlConstants.LOADINGLABEL_STRING_KEY = "LoadingLabel";
                UserSchemaControlConstants.WEBAPIREQUESTFAILED_ERROR = "WebApiRequestFailed";
                UserSchemaControlConstants.CONFIRMBUTTON_TEXT = "ConfirmButtonText";
            })(UserSchemaControlConstants = Controls.UserSchemaControlConstants || (Controls.UserSchemaControlConstants = {}));
        })(Controls = USD.Controls || (USD.Controls = {}));
    })(USD = MscrmControls.USD || (MscrmControls.USD = {}));
})(MscrmControls || (MscrmControls = {}));
